/**
 *  Demonstrate an allocator for string buffers that relies on hook objects.
 */
public class BufferAllocator
{

  /** Class that can be used to hook string 
   *  buffers into a singly-linked list.
   */
  class FreelistHook
  {
    FreelistHook next; 
    StringBuffer payload; 
  }
  
  private FreelistHook unusedHooks; 
  private FreelistHook freeList; 

  /**  No-arg constructor: Default to creating
   *   a free pool with 50 entries.
   */
  public BufferAllocator()
  {
    this(50); 
  }
  
  /**  Construct a pool of string buffers with the specified number of entries.
   *   @param num  Number of entries in the pool.
   */
  public BufferAllocator(int num)
  {
    for(int i=0;i<num;++i) freeHook(new FreelistHook()); 
    for(int i=0;i<num;++i) free(new StringBuffer()); 
  }
  
  /**  Return an unused hook to the list of free hooks. */
  private void freeHook(FreelistHook hook)
  {
    hook.next = unusedHooks; 
    unusedHooks = hook; 
  }
  
  /**  Get a hook from the free hook list. */
  private FreelistHook getHook()
  {
    FreelistHook hook = unusedHooks; 
    if(hook!=null) unusedHooks = hook.next; 
    return hook; 
  }
  

  /**  Return a string buffer to the free pool. 
   *   Note that this method does not verify that this string buffer
   *   originated in this pool.  (And that is OK, but could be confusing.)
   *   @param node A reference to the string buffer to be placed in the 
   *               free pool.
   */
  public synchronized void free(StringBuffer node)
  {
    FreelistHook hook = getHook(); 
    if(hook==null) throw new RuntimeException("Too few hooks"); 
    hook.payload = node; 
    hook.next = freeList; 
    freeList = hook; 
  }
  
  /**  Allocate a string buffer from the free pool.
   *   @return A reference to the "new" string buffer.
   */
  public synchronized StringBuffer allocate()
  {
    FreelistHook hook = freeList; 
    freeList = hook.next; 
    StringBuffer node = hook.payload; 
    freeHook(hook); 
    if(node!=null) node.setLength(0); 
    return node; 
  }
  
  /**	Exercise the buffer allocator */

  static public void main(String [] args)
  {
    BufferAllocator test = new BufferAllocator(50); 
    StringBuffer[] test2 = new StringBuffer[50]; 
    
    // Grab 30 StringBuffers from the allocator
    // And put a number in each StringBuffer
    for(int i=0;i<30;++i){
      test2[i] = test.allocate(); 
      test2[i].append(i); 
    }
    
    // Print 20 allocated StringBuffers
    for(int i=0;i<20;++i) System.out.print(test2[i] + " "); 
    // Return the String buffers to the free pool
    for(int i=0;i<30;++i) test.free(test2[i]); 
    // Grab 30 StringBuffers from the allocator
    // And put a number in each StringBuffer
    // (If the free function failed, there won't
    // be enough to satisfy these allocations
    for(int i=0;i<30;++i){
      test2[i] = test.allocate(); 
      test2[i].append(i); 
    }
    System.out.println(); 
    // Print 20 allocated StringBuffers
    for(int i=10;i<30;++i) System.out.print(test2[i] + " "); 
    System.out.println(); 
  }
}
