import javax.realtime.*; 
/**
 *  Demonstrate a subclass of AysnchEventHandler that
 *  can be reused.  This makes it suitable for use in
 *  a pool.
 *  This class overrides the AsyncEventHandler methods
 *  that it must to give behavior just like an AEH, but
 *  capable of being reset and reused.
 */
public class ReusableAEH extends AsyncEventHandler
{
  /**
   * A link used to put this object
   */
  public ReusableAEH next; 
  private Runnable logic; // The resettable logic
  
  public ReusableAEH()
  {
    super(); 
  }
  
  public ReusableAEH(boolean nonHeap)
  {
    super(nonHeap); 
  }
  
  public ReusableAEH(boolean nonHeap, Runnable logic)
  {
    // Note: Since we override handleAsyncEvent
    // passing a logic parameter to the constructor
    // has no effect.
    super(nonHeap); 
    setLogic(logic); 
  }
  
  public ReusableAEH(Runnable logic)
  {
    setLogic(logic); 
  }
  
  public ReusableAEH(SchedulingParameters scheduling,
    ReleaseParameters release,
    MemoryParameters memory,
    MemoryArea area,
    ProcessingGroupParameters group,
    boolean nonHeap,
    Runnable logic)
  {
    super(scheduling, release,
          memory, area, group, nonHeap,
          logic); // Note: Passing in logic is harmless
    setLogic(logic); 
  }
  
  // ... and other constructors matching all the
  // AEH constructors.
  
  public ReusableAEH(SchedulingParameters scheduling,
    ReleaseParameters release,
    MemoryParameters memory,
    MemoryArea area,
    ProcessingGroupParameters group,
    boolean nonHeap)
  {
    super(scheduling, release,
          memory, area, group, nonHeap, null); 
  }
  
  public ReusableAEH(SchedulingParameters scheduling,
    ReleaseParameters release,
    MemoryParameters memory,
    MemoryArea area,
    ProcessingGroupParameters group)
  {
    super(scheduling, release, memory, area, group, null); 
  }
  
  public ReusableAEH(SchedulingParameters scheduling,
    ReleaseParameters release,
    MemoryParameters memory,
    MemoryArea area,
    ProcessingGroupParameters group,
    Runnable logic)
  {
    super(scheduling, release,
           memory, area, group, logic); 
    setLogic(logic); 
  }
  
  /**
   * Change the logic executed by this AEH
   */
  public void setLogic(Runnable logic)
  {
    this.logic = logic; 
  }
  
  /**
   * Call the runnable logic
   */
  public void handleAsyncEvent()
  {
    if(this.logic!=null) this.logic.run(); 
  }
  
  public static void main(String [] args)
  {
    // Make a reusable AEH
    ReusableAEH rAEH = new ReusableAEH(); 
    
    // Make a source of events and attach
    // it to our reusable AEH
    PeriodicTimer timer = new PeriodicTimer(
                            null, // start now
                            new RelativeTime(1000, 0), // tick once per second
                            rAEH); 
    
    // Associate a Runnable with the AEH
    rAEH.setLogic(new Runnable() {
      public void run(){
        System.out.println("tick"); 
      }
    }
    ); 
    
    timer.start(); 
    
    // Let the AEH print ticks for a while
    try{
      Thread.sleep(10000); 
    }catch(Exception e){}
    
    // Attach a new Runnable that prints tocks
    rAEH.setLogic(new Runnable() {
      public void run(){
        System.out.println("Tock"); 
      }
    }
    ); 
    
    // Now let it print tocks for a while
    try{
      Thread.sleep(10000); 
    }catch(Exception e){}
    
    // Unhook the AEH from the timer
    timer.removeHandler(rAEH); 
    System.exit(0); 
  }
}
