import javax.realtime.*; 
/** ReusableThread is not a subclass of RealtimeThread, but it acts a lot like one,
 *  and with some patient addition of methods, it could act exactly 
 *  like a RealtimeThread.
 *  The "trick" is that a ReusableThread  contains a reference to 
 *  RealtimeThread with a special run method.
 *  Basically, ReusableThread has every method in Thread or RealtimeThread,
 *  and passes most of the calls right through to the client thread.
 */
public class ReusableThread implements Schedulable
{
  /**
   * A link field for the free list
   */
  public ReusableThread next = null; 
  
  /**
   * The hidden client thread.
   */
  private RealtimeThread innerThread; 
  
  public ReusableThread()
  {
    create(null, null, null, null, null, go); 
  }
  
  public ReusableThread(SchedulingParameters sched)
  {
    create(sched, null, null, null, null, go); 
  }
  
  public ReusableThread(SchedulingParameters sched,
      ReleaseParameters release)
  {
    create(sched, release, null, null, null, go); 
  }
  
  public ReusableThread(SchedulingParameters sched,
      ReleaseParameters release,
      MemoryParameters memory,
      MemoryArea area,
      ProcessingGroupParameters group,
      Runnable logic)
  {
    create(sched,
           release,
           memory,
           area,
           group,
           logic); 
  }
 
  /**  Create the client thread for this reusable thread. */ 
  private void create(SchedulingParameters sched,
      ReleaseParameters release,
      MemoryParameters memory,
      MemoryArea area,
      ProcessingGroupParameters group,
      Runnable logic)
  {
    if(logic!=null) go = logic; 
    innerThread = new RealtimeThread(sched,
                                     release,
                                     memory,
                                     area,
                                     group, this); 
  }
  

 /* The following are a partial set of the thread or real-time
  * thread methods that can simply be passed through to the client thread.
  */
  public boolean addIfFeasible()
  {
    return innerThread.addIfFeasible(); 
  }
  
  public boolean addToFeasibility()
  {
    return innerThread.addToFeasibility(); 
  }
  
  public void deschedulePeriodic()
  {
    innerThread.deschedulePeriodic(); 
  }
  
  public MemoryParameters getMemoryParameters()
  {
    return innerThread.getMemoryParameters(); 
  }
  
  public ProcessingGroupParameters getProcessingGroupParameters()
  {
    return innerThread.getProcessingGroupParameters(); 
  }
  
  public ReleaseParameters getReleaseParameters()
  {
    return innerThread.getReleaseParameters(); 
  }
  
  public Scheduler getScheduler()
  {
    return innerThread.getScheduler(); 
  }
  
  public SchedulingParameters getSchedulingParameters()
  {
    return innerThread.getSchedulingParameters(); 
  }
  
  public void interrupt()
  {
    innerThread.interrupt(); 
  }
  
  public boolean removeFromFeasibility()
  {
    return innerThread.removeFromFeasibility(); 
  }
  
  public void schedulePeriodic()
  {
    innerThread.schedulePeriodic(); 
  }
  
  public boolean setIfFeasible(
      ReleaseParameters release,
      MemoryParameters memory)
  {
    return innerThread.setIfFeasible(release, memory); 
  }
  
  public boolean setIfFeasible(
      ReleaseParameters release,
      MemoryParameters memory,
      ProcessingGroupParameters group)
  {
    return innerThread.setIfFeasible(release,
                                     memory,
                                     group); 
  }
  
  public boolean setIfFeasible(
      ReleaseParameters release,
      ProcessingGroupParameters group)
  {
    return innerThread.setIfFeasible(release, group); 
  }
  
  public void setMemoryParameters(
      MemoryParameters parameters) throws IllegalThreadStateException
  {
    innerThread.setMemoryParameters(parameters); 
  }
  
  public boolean setMemoryParametersIfFeasible(
      MemoryParameters parameters) throws IllegalThreadStateException
  {
    return innerThread.setMemoryParametersIfFeasible(parameters); 
  }
  
  public void setProcessingGroupParameters(
      ProcessingGroupParameters group)
  {
    innerThread.setProcessingGroupParameters(group); 
  }
  
  public boolean setProcessingGroupParametersIfFeasible(
      ProcessingGroupParameters group)
  {
    return innerThread.setProcessingGroupParametersIfFeasible(group); 
  }
  
  public void setReleaseParameters(ReleaseParameters release)
  {
    innerThread.setReleaseParameters(release); 
  }
  
  public boolean setReleaseParametersIfFeasible(ReleaseParameters release)
  {
    return innerThread.setReleaseParametersIfFeasible(
    release); 
  }
  
  public void setScheduler(Scheduler scheduler,
      SchedulingParameters scheduling,
      ReleaseParameters release,
      MemoryParameters memory,
      ProcessingGroupParameters group) throws
          IllegalThreadStateException
  {
    innerThread.setScheduler(scheduler,
                             scheduling,
                             release,
                             memory,
                             group); 
  }
  
  public void setScheduler(Scheduler scheduler) throws IllegalThreadStateException
  {
    innerThread.setScheduler(scheduler); 
  }
  
  public void setSchedulingParameters(
      SchedulingParameters scheduling) throws
          IllegalThreadStateException
  {
    innerThread.setSchedulingParameters(scheduling); 
  }
  
  public boolean setSchedulingParametersIfFeasible(
      SchedulingParameters scheduling) throws
          IllegalThreadStateException
  {
    return innerThread.setSchedulingParametersIfFeasible(scheduling); 
  }
  
  private boolean keepAlive = true; 
  private boolean active = false; 
  private boolean started = false; 
  private boolean waitForStart = true; 
  private Runnable go = null; 
  private Object lock = new Object();
  private int exitCt = 0; 
  
  /**
   * This will be the run method for the client thread
   */
  public void run()
  {
    while(keepAlive){
      // active is false
      
      synchronized(lock){
        // Wait until start() releases us to run
        // or end() releases us to exit
        try{
          while(waitForStart&&keepAlive) lock.wait(); 
          if(!keepAlive) break; 
        }catch(InterruptedException e){
          continue; 
        }finally{
          active = true; 
          waitForStart = true; 
        }
      }
      // end sync
      // active is true
      // Since start will throw an exception if it is
      // called while active is true, and it is the only
      // other method that writes waitForStart,
      // waitForStart is also true.
      
      try{
        if(go==null) 
          dummyGo(); 
        else 
          go.run(); // Run the payload in this thread
      }catch(Exception e){
        // Discard all exceptions
      }catch(Error t){
        keepAlive = false; 
        throw t; 
      }finally{
        // Wake anyone waiting in join
        synchronized(lock){
          active = false; 
          exitCt += 1; 
          lock.notifyAll(); 
        }
      }
    }
  }
  
  /**
   *  Have the representative logic use this method if the "logic" 
   *  object is still unspecified
   */
  private void dummyGo()
  {
    return; 
  }
  
  /**
   * Let the run method return next time through the loop.
   */
  public void end()
  {
    synchronized(lock){
      keepAlive = false; 
      lock.notifyAll(); 
    }
  }
  
  /**
   * wait for the run method to complete a task
   */
  public void join() throws InterruptedException
  {
    int lastExitCt = exitCt; 
    synchronized(lock){
      if(!keepAlive) return; 
      while(exitCt==lastExitCt) lock.wait(); 
    }
  }
  
  /**
   * If the run method is executing a task or about
   * to execute a task, wait for it to complete.
   */
  public void joinIfRunning() throws InterruptedException
  {
    int lastExitCt = exitCt; 
    if(active||!waitForStart) join(); 
  }
  
  /**
   * Is the rt thread executing its payload?
   */
  public boolean isActive()
  {
    return active; 
  }
  
  /** If the client thread is alive but has finished running the last
   *  logic, (or has not yet be given logic to run) associate the new logic
   *  with the client thread and release it.
   */
  public void start(Runnable logic)
  {
    synchronized(lock){
      if(active) throw new
      RuntimeException("starting a running thread"); 
      if(!keepAlive) 
        throw new
      RuntimeException("starting a dead thread"); 
      go = logic; 
      if(!started){
        started = true; 
        innerThread.start(); 
      }
      
      // Release the RT thread.
      // It must be waiting or active would not be false
      waitForStart = false; 
      lock.notifyAll(); 
    }
  }
  
  public static void main(String [] args)
  {
    ReusableThread experiment = new ReusableThread(); 
    experiment.start(new Runnable() {
      public void run(){
        for(int i=0;i<5;++i){
          System.out.print(" 1 "); 
          try{
            Thread.sleep(1000); 
          }catch(Exception e){}
        }
        System.out.println(); 
      }
    }
    ); 
    
    try{
      experiment.joinIfRunning(); 
    }catch(InterruptedException e){
      System.out.println("Interrupted in join"); 
    }
    
    experiment.start(new Runnable() {
      public void run(){
        for(int i=0;i<5;++i){
          System.out.print(" 2 "); 
          try{
            Thread.sleep(2000); 
          }catch(Exception e){}
        }
        System.out.println(); 
      }
    }
    ); 
    try{
      experiment.joinIfRunning(); 
    }catch(InterruptedException e){}
    experiment.end(); 
  }
}
