import javax.realtime.*; 
/**
 * Extend an RTSJ default priority scheduler with
 * admission control/feasibility services.
 */
public class FPriorityScheduler extends PriorityScheduler
{
  private double utilization = 0; 
  private final static String policy = "PriortySchedulerWithFeasibility"; 
  private static FPriorityScheduler me = null; 
  
  /**
   * The only constructor for this class is private.
   * To get an instance of FPriorityScheduler use the
   * instance method
   */
  private FPriorityScheduler()
  {
    super.instance(); 
  }
  
  /**
   * Add a schedulable (thread or AEH) to the feasible set.
   * 
   * Warning, this method will happily add the same thread to the
   * feasible set many times.
   * 
   * Note, this class only tracks the total CPU consumption
   * off all registered schedulable entities.
   * 
   * @param s The schedulable (rt thread or AEH) to add to
   * the feasible set.
   */
  protected synchronized boolean  addToFeasibility(Schedulable s)
  {
    if(!check(s)) return false;
    PeriodicParameters param = (PeriodicParameters)
        s.getReleaseParameters(); 
    RelativeTime cost = param.getCost(); 
    if(cost==null) 
      cost = param.getDeadline(); 
    utilization += calcUtilization(cost, param.getPeriod()); 
    return utilization <= 1.0;
  }
  
  /**
   * Calculate the CPU utilization given cost and period.
   * @param cost the cost of a schedulable
   * @param period the period of the schedulable
   * @return the CPU utilization for this schedulable
   */
  private double calcUtilization(RelativeTime cost,
  RelativeTime period)
  {
    double costMicros = (cost.getMilliseconds() * 1000.0) +
        (cost.getNanoseconds() / 1000.0); 
    double periodMicros = (period.getMilliseconds() * 1000.0) +
        (period.getNanoseconds() / 1000.0); 
    return costMicros / periodMicros; 
  }
  
  /**
   * Check a Schedulable's eligibility for admission control
   * @param s The Schedulable to test
   * @return  True if the schedulable is eligible
   */
  private boolean check(Schedulable s)
  {
    if(s.getReleaseParameters()instanceof PeriodicParameters){
      PeriodicParameters param = (PeriodicParameters)
          s.getReleaseParameters(); 
      if(!positive(param.getPeriod())) 
        throw new IllegalArgumentException("period <= 0"); 
      if(param.getCost()==null&& param.getDeadline()==null) 
        return false; 
      return true; 
    }else 
      return false; 
  }
  
  /**
   * Return the name of the policy this scheduler implements.
   * 
   * @return The name of this policy
   * "Priority Scheduler with Feasibility"
   */
  
  public java.lang.String getPolicyName()
  {
    return policy; 
  }
  
  /**
   * Return a reference to the singleton instance of this class.
   * If the class has not yet been instanciated, construct it.
   * 
   * @return A reference to an instance of class
   * FPriorityScheduler
   */
  
  public static PriorityScheduler instance()
  {
    if(me==null) 
      me = new FPriorityScheduler(); 
    return me; 
  }
  
  /**
   * Return true if the set of schedulables that have been through
   * addToFeasible, but not removeFromFeasible do not use more
   * than 100% of CPU time
   * 
   * @return True if the load is feasible, false otherwise
   */
  
  public boolean isFeasible()
  {
    return utilization <= 1.0; 
  }
  
  /**
   * Change the scheduling attributes of the specified
   * schedulable.
   * 
   * 
   * WARNING: This method assumes that the schedulable entity is
   * already feasible
   * 
   * @param schedulable A reference to a rt thread or AEH that
   * should be changed
   * @param release The release parameters that should be
   * tested for feasibility
   * @param memory The memory parameters that should be
   * tested for feasibility
   * @return true if the change is feasible, false otherwise
   */
  public synchronized boolean changeIfFeasible(
      Schedulable schedulable,
      ReleaseParameters release,
      MemoryParameters memory)
  {
    RelativeTime cost; 
    double oldUtil; 
    double newUtil; 
    if(!check(schedulable)) //  Check schedulable's attributes
      return false; // Don't add ill-formed schedulables
    if(release instanceof PeriodicParameters){
      PeriodicParameters param = (PeriodicParameters)
          schedulable.getReleaseParameters(); 
      cost = param.getCost(); 
      if(cost==null) 
        cost = param.getDeadline(); 
      oldUtil = calcUtilization(cost, param.getPeriod()); 
      param = (PeriodicParameters)release; 
      if(!positive(param.getPeriod())) 
        throw new IllegalArgumentException("period <= 0"); 
      
      cost = param.getCost(); 
      if(cost==null) 
        cost = param.getDeadline(); 
      newUtil = calcUtilization(cost, param.getPeriod()); 
      if(utilization+newUtil-oldUtil<=1.0){
        utilization += (newUtil - oldUtil); 
        return true; 
      }else 
        return false; 
    }else{
      // Not PeriodicParameters
      throw new IllegalArgumentException("release must be periodic"); 
    }
  }
  
  /**
   * Remove the specified schedulable entity from
   * feasibility analysis.
   * 
   * WARNING: This method does not check that the schedulable is
   * current in the feasible set.
   * 
   * @param s A reference to a schedulable entity
   */
  protected synchronized boolean removeFromFeasibility(
  Schedulable s)
  {
    if(!check(s)) 
      return false; 
    PeriodicParameters param = (PeriodicParameters)s.getReleaseParameters(); 
    RelativeTime cost = param.getCost(); 
    if(cost==null) 
      cost = param.getDeadline(); 
    utilization -= calcUtilization(cost, param.getPeriod()); 
    return utilization <= 1.0;
  }
  
  /**
   * Test for relative time greater than 0
   * @param t RelativeTime value
   * @return true iff time > 0
   */
  
  private boolean positive(RelativeTime t)
  {
    if(t.getMilliseconds()==0) 
      return t.getNanoseconds() > 0; 
    else 
      return t.getMilliseconds() > 0; 
  }
  
  /**
   * main method to test this class
   */
  static public void main(String [] args)
  {
    RealtimeThread rtt = new RealtimeThread(
        null, // Default sceduling Parameters
        new PeriodicParameters(
            null, // Begin running at start()
            new RelativeTime(250, 0), // Period is 1/4 sec
        new RelativeTime(25, 0), // Cost is 1/40 sec
        new RelativeTime(200, 0), // Deadline is 1/5 sec
        null, // No overrun handler
        null), // No miss handler
        null, // Default memory parameters
        null, // Default memory area
        null, // Default processing group
        new Runnable (){ // Logic
          public void run() {
            System.out.println("Running " +
            RealtimeThread.currentRealtimeThread().
            getScheduler().getPolicyName()); 
          }
        }
    ); 
    rtt.setScheduler(FPriorityScheduler.instance()); 
    rtt.start(); 
    try{
      rtt.join(); 
    }catch(Exception e){}
  }
}
