import javax.realtime.*; 

/**
 * Maintain a map of memory areas and names
 */

public class IdMemArea
{
  // Since these are static, they are allocated in immortal memory
  private static char [][] name = new char[50][16]; 
  private static int [] area = new int[50]; 
  static private char [] nullMem = {'N', 'u', 'l', 'l', ' ', 'm', 'e', 'm', 'o', 'r', 'y', ' ', 'a', 'r', 'e', 'a'};
  
  static private char [] immortalMem ={'I', 'm', 'm', 'o', 'r', 't', 'a', 'l'};
  
  static private char [] heapMemory = {'H', 'e', 'a', 'p'};
  
  static private char [] unknownMem = {'U', 'n', 'k', 'n', 'o', 'w', 'n', ' ', 'm', 'e', 'm', ' ', 'a', 'r', 'e', 'a'};
  
  /**
   * Add a mapping.
   * This method has minimal error checking. It will happily add a
   * memory area many times, or use the same name for multiple memory
   * areas.  USER BEWARE.
   * Three memory areas cannot be mapped: null, heap, and immortal.
   * Any attempt to assign a name to one of these silently does nothing.
   * All three of these areas have default names, and we do not permit
   * overrides.
   * @param mem The new memory area
   * @param nameStr The name that should be used for the new area.
   */
  static void addNew(final MemoryArea mem, final String nameStr)
  {
    int i, j; 
    if(mem==null) return; 
    if(mem==HeapMemory.instance()) return; 
    if(mem==ImmortalMemory.instance()) return; 
    for(i=0;i<50;++i){
      if(area[i]==0){
        // We don't store an actual reference to the memory area
        // just a hash.  This vastly simplifies thing.
        // (Particularly the possibility that the memory area
        // might be allocated in a scope.  We would get an exception
        // from trying to store that in the static (==immortal)
        // array.
        area[i] = mem.hashCode(); 
        for(j=0;j<16;++j) name[i][j] = ' '; 
        for(j=0;j<nameStr.length()&&j<16;++j) name[i][j] = nameStr.charAt(j); 
        break; 
      }
    }
  }
  
  /**
   * Remove the mapping from the designated memory area.
   * @param mem The memory area to unmap.
   */
  static void remove(MemoryArea mem)
  {
    int i; 
    for(i=0;i<50;++i){
      if(area[i]==mem.hashCode()){
        area[i] = 0; 
        break; 
      }
    }
  }
  
  static private char [] idArea1(MemoryArea mem)
  {
    int i; 
    if(mem==null){
      return nullMem; 
    }else if(mem==HeapMemory.instance()){
      return heapMemory; 
    }else if(mem==ImmortalMemory.instance()){
      return immortalMem; 
    }else{
      for(i=0;i<50;++i){
        if(area[i]==mem.hashCode()){
          return name[i]; 
        }
      }
    }
    return unknownMem; 
  }
  
  /**
   * Return the name associated with the specified memory area.
   * @param mem A memory area
   * @return The String name associated with the memory area, or
   * "unknown mem area" if the memory area cannot be identified.
   */
  static String idArea(MemoryArea mem)
  {
    int i; 
    
    if(mem==null) return "Null memory area"; 
    if(mem==HeapMemory.instance()) return "Heap"; 
    else if(mem==ImmortalMemory.instance()) return "Immortal"; 
    else for(i=0;i<50;++i){
      if(area[i]==mem.hashCode()) return new String(name[i]); 
    }
    
    return "unknown mem area"; 
  }
  
  /**
   * Return the name of the memory area an object is in.
   * Note, if a memory area is passed to this method, it returns the
   * memory area the memory area is in, not the memory area's name.
   * @param mem The memory area to identify.
   * @return The String name for the memory area containing the object.
   */
  static String id(Object obj)
  {
    if(obj==null) return "Null object"; 
    MemoryArea mem = MemoryArea.getMemoryArea(obj); 
    return idArea(mem); 
  }
  
  /**
   * Return the name of the current thread's default memory allocation
   * context.
   * @return The current default memory allocation area's name.
   */
  static String current()
  {
    return idArea(RealtimeThread.getCurrentMemoryArea()); 
  }
  
  /**
   * Return the names of the scopes in the current scope stack.
   * Creates in the current allocation context
   * the StringBuffer it will return.
   * @return A StringBuffer containing the names of the memory
   * areas in the scope stack concatenated with colons between them.
   */
  static StringBuffer scopeStack()
  {
    try{
      int size = 0; 
      int n = RealtimeThread.getMemoryAreaStackDepth(); 
      size = (n - 1) * 2; 
      
      for(int i=0;i<n;++i){
        MemoryArea mem = RealtimeThread.getOuterMemoryArea(i); 
        size += characters(idArea1(mem)); 
      }
      StringBuffer buffer = new StringBuffer(size); 
      
      for(int i=n-1;i>=0;--i){
        if(i!=n-1) buffer.append(": "); 
        MemoryArea mem = RealtimeThread.getOuterMemoryArea(i); 
        buffer.append(idArea1(mem)); 
      }
      return buffer; 
    }catch(Throwable e){
      System.out.println("Exception in scopeStack"); 
      e.printStackTrace(); 
      return new StringBuffer("X"); 
    }
  }
  
  static private int characters(char [] s)
  {
    int n; 
    for(n=0;n<s.length;++n) if(s[n] == 0) return n; 
    return n; 
  }

}

